clear all
clc
%
% This Program Evaluates the Process Hazard Matrix (PHM)
% For each process, evaluates the General Process Hazard and Special
% Process Hazard, the Process Hazard Matrix comes from the combination 
% of the Explosion Index and the Toxicity Index.  
% The Goods Risk Factor (GRF) evaluated using NFPA classification
%
% -------------------------------------------------------------------------
% Version          : 1
% Programmed by    : Kostas Kostaras
% -------------------------------------------------------------------------
%
% Initial Inputs is the matrix InData(n,m) where:
% InData(n,1)  = Process number
% InData(n,2)  = Reactivity Material Hazard, using the Adiabatic
%                Decomposition Temperature with range [500, 1000] degC
% InData(n,3)  = Flammability Material Hazard, using Flash Point
%                Temperature with range [0, 100] degC
% InData(n,4)  = Virulence - Toxicity Material Factor, using LD50 with
%                range  [0, 2000] mg/Kg
% InData(n,5)  = Percentage Allowable Concentration of Material in Process,
%                range [0, 100]     
% InData(n,6)  = InputGPPH(n,1): the Heat transfer of the reaction that
%                takes place with
%                range: Low    = [0.0 0.5]
%                       Medium = [0.1 0.9]
%                       High   = [0.5 1.0]
% InData(n,7)  = InputGPPH(n,2): the type of the reaction
%                range: Endothermic  = [-1 0]
%                       Exothermic   = [0  1]
% InData(n,8)  = InputGPPH(n,4): percent needs for handling or storage of
%                goods according to the process
%                range: LowHandlingStorage      = [0 50]
%                       MediumHandlingStorage   = [25 50 75]
%                       HighHandlingStorage     = [50 100]
% InData(n,9)  = IndataOFC(n,2): the process temperature degC
% InData(n,10) = IndataOFC(n,3): the process pressure atm
% InData(n,11) = InputSPH2(n,2): the procedure leakage with range [0 100]
%                percent by the historical knowledge 
% InData(n,12) = InputSPH2(n,3): the corrosion and erosion in mm/year with
%                range [0 1]
% InData(n,13) = Is the minimum acceptable concentration (Mac Value) of the
%                goods
%
% -------------------------------------------------------------------------
% ----------------   INSERT DATA  -----------------------------------------
% ---------------- FROM EXCEL FILE INDATA.XLS -----------------------------
% -------------------------------------------------------------------------
%Indata=[1 900 10 500 80 0.5 -1 25 650 2 0 0.1 4
%        2 750 90 500 80 0.7 1 75 400 1 30 0.5 35
%        3 500 90 500 80 0.9 -1 100 1000 1 70 1 100];
%
Indata=xlsread('INDATA',1);
% -------------------------------------------------------------------------
% -------------------------------------------------------------------------
%
[n1,m1]=size(Indata);
%
% -------------------------------------------------------------------------
% This part of the program evaluates Goods Risk Factor (GRF) for each unit
% of the Process, using a fuzzy controller named as GoodsFuzzyControl.fis
%
% IndataGRF(n,1)  = InData(n,2): Reactivity Material Hazard, using the
%                   Adiabatic Decomposition Temperature with range 
%                   [500, 1000] degC
% IndataGRF(n,2)  = InData(n,3): Flammability Material Hazard, using Flash
%                   Point Temperature with range [0, 100] degC 
% IndataGRF(n,3)  = InData(n,4):Virulence - Toxicity Material Factor, using
%                   LD50 with range  [0, 2000] mg/Kg
% IndataGRF(n,4)  = InData(n,5):Percentage Allowable Concentration of
%                   Material in Process, range [0, 100]
%
% OutGRF(n,1) = Reactivity Material Hazard 
% OutGRF(n,2) = Flammability Material Hazard
% OutGRF(n,3) = Virulence - Toxicity Material Factor
% OutGRF(n,4) = Material Factor (MF) of the good
%
%
for i1=1:n1
    for j1=2:5
IndataGRF(i1,j1-1)=Indata(i1,j1);
    end
end
%
% In the matrix below is the initial data for each process of the plant
%
' In the matrix below is the initial data for each process of the plant '
IndataGRF
%
' In the matrix below each line is a process goods classification according the NFPA ';
' OutGRF(n,1) = Reactivity Material Hazard ';
' OutGRF(n,2) = Flammability Material Hazard ';
' OutGRF(n,3) = Virulence - Toxicity Material Factor ';
' OutGRF(n,4) = MF';
%
OutGRF=evalfis(IndataGRF, readfis('GoodsFuzzyControl'))
%
for i7=1:n1
    OutGRF123(i7,1)=OutGRF(i7,1);
    OutGRF123(i7,2)=OutGRF(i7,2);
    OutGRF123(i7,3)=OutGRF(i7,3);
 end
OutGRF123
%
GRF=max(OutGRF123,[],2);
%
figure (1)
%
subplot(2,1,1)
bar(OutGRF123)
%% Create axes
title('NFPA Goods Fuzzy Classification');
xlabel('Process');
ylabel('Class');
legend({'Reactivity','Flamability','Toxicity'},'Location','NorthEastOutside');
%
subplot(2,1,2)
bar(GRF)
%% Create axes
title('Fuzzy Goods Risk Factor');
xlabel('Process');
ylabel('Class');
%
% The program creates the Process Goods Risk Factor Matrix (PGRFM(n,2)) where:
% PGRFM(n,1): Process number
% PGRFM(n,2): GRF for the specific process
%
for i2=1:n1
PGRFM(i2,1)=Indata(i2,1);
PGRFM(i2,2)=OutGRF123(i2,1);
PGRFM(i2,3)=OutGRF123(i2,2);
PGRFM(i2,4)=OutGRF123(i2,3);
PGRFM(i2,5)=GRF(i2);
end
%
' In the Matrix below the first column is process and the second is the Hazard of the goods in the process'
' PGRFM(n,1) = the process, PGRFM(n,2) = the maximum risk factor of the process ' 
%
PGRFM
%
% -------------------------------------------------------------------------
% The program calls the fuzzy controler 
% GeneralProcessHazardFuzzyControl.fis
% to evaluate the General Process Hazard penalties come from the reaction
% that takes place and the handling or the storage of goods The program
% evaluates the General Process Hazard using us inputs the matrix
%
% InputGPH(n,4) where n = process number and:
%
% InputGPPH(n,1) = InData(n,6): the Heat transfer of the reaction that takes
%                  place with
%                  range: Low   = [0.0 0.5]
%                  Medium       = [0.1 0.9]
%                  High         = [0.5 1.0]
% InputGPPH(n,2) = InData(n,7): the type of the reaction
%                  range: Endothermic  = [-1 0]
%                  Exothermic          = [0  1]
% InputGPPH(n,3) = PGRFM(n,2) : the goods risk factor GRF
%                  range: LowGoodsRiskFactor = [0 1.5]
%                  MediumGoodsRiskFactor     = [0.75 1.5 2.25]
%                  HighGoodsRiskFactor       = [1.5 3]
% InputGPPH(n,4) = InData(n,8): percent needs for handling or storage of
%                  goods according to the process
%                  range: LowHandlingStorage = [0 50]
%                  MediumHandlingStorage     = [25 50 75]
%                  HighHandlingStorage       = [50 100]
%
% The Outputs of the controller are:
%
% OutGPHP(n,1)  = The reaction penalty with range [0 1.5]
% OutGPHP(n,2)  = The handling storage penalty with range  [0 0.8]
%
for i1=1:n1
InputGPPH(i1,1)=Indata(i1,6);
InputGPPH(i1,2)=Indata(i1,7);
InputGPPH(i1,3)=PGRFM(i1,2);
InputGPPH(i1,4)=Indata(i1,8);
end
%
InputGPPH;
%
' The matrix below shows the general process hazard penalties comes from reaction and from handling storage '
%
OutGPHP=evalfis(InputGPPH, readfis('GeneralProcessHazardFuzzyControl'))
%
% -------------------------------------------------------------------------
% The program evaluates the Special Process Hazard Penalties come
% from process temperature, process pressure (OperationFuzzyControl.fis)
% also come from leakage of goods and corrosion-erosion of goods (SpecialProcessHazard2.fis) 
% to run the OperationFuzzyControl.fis the program needs:
%
% GRF(n)       = IndataOFC(n,1): the goods risk factor
% InData(n,9)  = IndataOFC(n,2): the process temperature degC
% InData(n,10) = IndataOFC(n,3): the process pressure atm
%
% The outputs of the fuzzy controller are
%
% OutOFC(n,1)  = the pressure process penalty with range [0 2] 
% OutOFC(n,2)  = the temperature process penalty with range [0 1]
%
for i3=1:n1
    IndataOFC(i3,1)=GRF(i3);
    if Indata(i3,9)>Indata(i3,3);
    IndataOFC(i3,2)=(abs(Indata(i3,9))-abs(Indata(i3,3)))/abs(Indata(i3,9));
    else 
    IndataOFC(i3,2)=(abs(Indata(i3,9))-abs(Indata(i3,3)))/abs(Indata(i3,3)); 
    end
    IndataOFC(i3,3)=(Indata(i3,10)-1)/Indata(i3,10);
end
IndataOFC
%
' The matrix below shows the penalties come from the process temperature ant the process pressure '
%
OutOFC=evalfis(IndataOFC, readfis('SpecialProcessHazards1')) 
%
% -------------------------------------------------------------------------
% The program evaluates the Special Process Hazard Penalties come from
% leakage of goods and corrosion-erosion of goods
% (SpecialProcessHazards2.fis) to run the SpecialProcessHazard2.fis 
% the program needs the inputs:
%
% GRF(n)       = InputSPH2(n,1): the goods risk factor
% Indata(n,11) = InputSPH2(n,2): the procedure leakage with range [0 100]
%                percent by the historical knowledge 
% Indata(n,12) = InputSPH2(n,3): the corrosion and erosion in mm/year with
%                range [0 1<]
%
% the Output of the SpecialProcessHazard2.fis are the penalties:
%
% OutputSPH2(n,1) = the penalty comes from the leakage with range [0 1.75]
% OutputSPH2(n,2) = the penalty comes from the corrosion - erosion with,
%                   range [0 0.5]
%
for i4=1:n1;
    InputSPH2(i4,1)=GRF(i4);
    InputSPH2(i4,2)=Indata(i4,11);
    InputSPH2(i4,3)=Indata(i4,12);
end
InputSPH2;
%
'The matrix beloe shows the penalties come from the leakage and the corrosion - erosion of the process ' 
%
OutputSPH2=evalfis(InputSPH2, readfis('SpecialProcessHazards2'))
%
% -------------------------------------------------------------------------
% Dedermination of the Explosion Index
% -------------------------------------------------------------------------
%
% ExplosionIndex = (Material Fctor)(1 + General Process Hazard)(1 + Special
%                  Process Hazard)
% ExplosionIdex = MF*(1+RP+HSP)*(1+PPP+PTP+LP+CEP)
% MF  = OutGRF(n,4) : Material Factor (MF) of the good, with range [0 40]
% From General Process Hazard
% RP  = OutGPHP(n,1): The reaction penalty, with range [0 1.5]
% HSP = OutGPHP(n,2): The handling storage penalty, with range  [0 0.8]
% From Special Process Hazard
% PPP = OutOFC(n,1): the pressure process penalty, with range [0 2]
% PTP = OutOFC(n,2): the temperature process penalty, with range [0 1]
% LP  = OutputSPH2(n,1): the penalty comes from the leakage, with range [0
%       1.75]
% CEP = OutputSPH2(n,2) = the penalty comes from the corrosion - erosion
%       with, range [0 0.5]
%
' The Explosion Index of the process is '
for i10=1:n1;
    MF(i10,1)=OutGRF(i10,4);
    GPHM(i10,1)=OutGPHP(i10,1)+OutGPHP(i10,2);
% Classification of General Process Hazard Penalties
if GPHM(i10,1)<0.575;
    GPHclass(i10,1)=0.;
elseif (GPHM(i10,1)>=0.575)&(GPHM(i10,1)<1.15);
GPHclass(i10,1)=1. ;
elseif (GPHM(i10,1)>=1.15)&(GPHM(i10,1)<1.725);
GPHclass(i10,1)=2. ;
elseif (GPHM(i10,1)>=1.725);
    GPHclass(i10,1)=3.;
end
%
    SPHM(i10,1)=OutOFC(i10,1)+OutOFC(i10,2)+OutputSPH2(i10,1)+OutputSPH2(i10,2);
% Classification of Special Process Hazard Penalties
if SPHM(i10,1)<1.3125;
    SPHclass(i10,1)=0.;
elseif (SPHM(i10,1)>=1.3125)&(SPHM(i10,1)<2.625);
SPHclass(i10,1)=1. ;
elseif (SPHM(i10,1)>=2.625)&(SPHM(i10,1)<3.9375);
SPHclass(i10,1)=2. ;
elseif (SPHM(i10,1)>=3.9375);
    SPHclass(i10,1)=3.;
end
%
figure (2)
grafclass=[GPHclass SPHclass];
bar(grafclass)
title('Classes of Process Hazard Penalties');
xlabel('Process');
ylim([0 3]);
ylabel('Class');
legend({'General Process Hazard','Special Process Hazard'},...
    'Location','NorthEastOutside');
% ----------------------------------------------------
% Data for contoler operation tests
%GPHM(i10,1)=2.0125;
%GPHclass(i10,1)=3.;
%SPHM(i10,1)=0.65625;
%SPHclass(i10,1)=0.;
%
% ----------------------------------------------------
    ExplosionIndexnum(i10,1)=OutGRF(i10,4)*(1.+GPHM(i10,1))*(1.+SPHM(i10,1));
%
    if ExplosionIndexnum(i10,1)<32.;
    ExplosionIndex(i10,1)=0.;
    elseif (ExplosionIndexnum(i10,1)>=32.)&(ExplosionIndexnum(i10,1)<107.);
    ExplosionIndex(i10,1)=1.;
    elseif (ExplosionIndexnum(i10,1)>=107.)&(ExplosionIndexnum(i10,1)<224.);
    ExplosionIndex(i10,1)=2.;
    elseif ExplosionIndexnum(i10,1)>=224.;
    ExplosionIndex(i10,1)=3.;
    end
end
%    
[PGRFM MF GPHM SPHM ExplosionIndex];
%
% -------------------------------------------------------------------------
% Dedermination of Toxicity Index
% -------------------------------------------------------------------------
% 
% ToxicityIndex = ((TH - TS)/100)*(1 + General Process Hazard + Special
%                 Process Hazard)
% ToxicityIndex=((TH - TS)/100)*(1 + GPH + SPH)
% TH  = Toxicity penalty, with range [0 325]
%       calculated by TH.fis
%       IndataTH(n,1) = OutGRF(n,3): Virulence - Toxicity Material Factor,
%                       with range [0, 4]
%       OutputTH(n,1) = TH(n,1): Toxicity Penalty, with range [0, 325]
% TS  = Mac Value Penalty, with range [50, 125]
%       calculated by Macvaluepenalty.m
%       IndataMacvalue(n,1) = InData(n,13)
%
for i11=1:n1;
IndataTH(i11,1)=OutGRF(i11,3);
IndataMacvalue(i11,1)=Indata(i11,13);
%
TS(i11,1)=Macvaluepenalty(IndataMacvalue(i11,1));
end
%
TH=evalfis(IndataTH, readfis('TH'));
' [ TH TS ] '
[TH TS]
%
' The Toxicity Index is ';
for i12=1:n1;
    ToxicityIndexnum(i12,1)=((TH(i12,1)-TS(i12,1))/100.)*(1.+GPHM(i12,1)+SPHM(i12,1));
%
    if ToxicityIndexnum(i12,1)<1.44375;
    ToxicityIndex(i12,1)=0.;
    elseif (ToxicityIndexnum(i12,1)>=1.44375)&(ToxicityIndexnum(i12,1)<3.58125);
    ToxicityIndex(i12,1)=1.;
    elseif (ToxicityIndexnum(i12,1)>=3.58125)&(ToxicityIndexnum(i12,1)<11.65938);
    ToxicityIndex(i12,1)=2.;
    elseif ToxicityIndexnum(i12,1)>=11.65938;
    ToxicityIndex(i12,1)=3.;
    end
end
ToxicityIndexnum;
[PGRFM MF GPHclass SPHclass ExplosionIndex ToxicityIndex];
% -------------------------------------------------------------------------
% 
% f = the final matrix with f(n,1) = process number, f(n,2) = the maximum
% hazard between explosion index and toxicity index
%
PHM=max([ExplosionIndex ToxicityIndex],[],2);
for i13=1:n1;
    f(i13,1)=i13;
    f(i13,2)=PHM(i13);
end
f;        
%
% In the matrix below is shown the Combination Process Hazard Matrix for
% the total processes of the plant. The program calls the CPHM(f) function 
%
' In the matrix below is shown the Combination Process Hazard Matrix for the total processes of the plant '
CPHM=CPHM(f)
%
OutputData=[PGRFM MF GPHclass SPHclass ExplosionIndex ToxicityIndex PHM] 
xlswrite('INDATA',OutputData,2,'A6');
xlswrite('INDATA',CPHM,3,'B3');

figure (3)
subplot(2,1,1)
grafclass2=[ExplosionIndex ToxicityIndex];
bar(grafclass2)
title('Explocition & Toxicity Index');
xlabel('Process');
ylim([0 4]);
ylabel('Class');
legend({'Explosion Index','Toxicity Index'},'Location','NorthEastOutside');
%
subplot(2,1,2)
bar(PHM)
title('Process Hazard Index');
xlabel('Process');
ylim([0 4]);
ylabel('Class');

%
figure (4)
pcolor(CPHM)
title('Color Combination Process Hazard Matrix')
xlabel('Process');
ylabel('Process');
colorbar;
